"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  RowID: () => RowID2,
  RowIDWithConfig: () => RowIDWithConfig,
  decode: () => decode2,
  default: () => src_default,
  encode: () => encode2,
  generate: () => generate2,
  getRandomness: () => getRandomness2,
  verify: () => verify2
});
module.exports = __toCommonJS(src_exports);

// src/common/configs.ts
var CHAR_LIST = "0123456789ABCDEFGHJKMNPQRSTVWXYZ";
var TIMESTAMP_LENGTH = 10;
var RANDOMNESS_LENGTH = 22;

// src/functions/common/decode.ts
var decode = /* @__PURE__ */ __name((opts) => {
  const ECD = "Encoded";
  if (typeof opts.encoded !== "string") {
    throw new TypeError(`${ECD} is not a string`);
  }
  if (opts.encoded.length < TIMESTAMP_LENGTH) {
    throw new TypeError(`${ECD} is not long enough to be decoded`);
  }
  if (!opts.encoded.match(new RegExp(`^[${opts.charList}]+$`))) {
    throw new TypeError(`${ECD} is not valid`);
  }
  const charList = opts.charList;
  const charListLength = charList.length;
  const encoded = opts.encoded.slice(0, TIMESTAMP_LENGTH).toUpperCase();
  let timestamp = 0;
  for (let i = 0; i < TIMESTAMP_LENGTH; i++) {
    timestamp = timestamp * charListLength + charList.indexOf(encoded[i]);
  }
  return new Date(Math.max(timestamp, 0));
}, "decode");

// src/functions/validateNumber.ts
var validateNumber = /* @__PURE__ */ __name((options) => {
  const maxInt = Number.MAX_SAFE_INTEGER ?? 9007199254740991;
  if (typeof options.number !== "number") {
    throw new TypeError(`${options.name} is not a number`);
  }
  if (options.number < 0) {
    throw new RangeError(`${options.name} must be greater than or equal to 0`);
  }
  if (options.number > maxInt) {
    throw new RangeError(`${options.name} must be less than or equal to ${maxInt}`);
  }
}, "validateNumber");

// src/functions/common/encode.ts
var encodeProcess = /* @__PURE__ */ __name((opts) => {
  const charList = opts.charList;
  const charListLength = charList.length;
  let encoded = "";
  let remaining = opts.timestamp;
  for (let i = 0; i < TIMESTAMP_LENGTH; i++) {
    encoded = charList[remaining % charListLength] + encoded;
    remaining = Math.floor(remaining / charListLength);
  }
  return encoded;
}, "encodeProcess");
var unsafedEncode = /* @__PURE__ */ __name((opts) => {
  return encodeProcess({
    timestamp: opts.timestamp,
    charList: opts.charList
  });
}, "unsafedEncode");
var encode = /* @__PURE__ */ __name((opts) => {
  validateNumber({
    name: "Timestamp",
    number: opts.timestamp
  });
  return encodeProcess({
    timestamp: opts.timestamp,
    charList: opts.charList
  });
}, "encode");

// src/functions/common/getRandomness.ts
var getRandomByte = /* @__PURE__ */ __name(() => {
  if (typeof window !== "undefined" && window.crypto && window.crypto.getRandomValues) {
    try {
      return window.crypto.getRandomValues(new Uint8Array(1))[0];
    } catch (e) {
    }
  }
  if (typeof require === "function") {
    try {
      return require("crypto").randomBytes(1)[0];
    } catch (e) {
    }
  }
  return Math.floor(Math.random() * 256);
}, "getRandomByte");
var randomnessProcess = /* @__PURE__ */ __name((opts) => {
  let randomness = "";
  for (let i = 0; i < opts.randomnessLength; i++) {
    randomness += opts.charList[getRandomByte() % opts.charList.length];
  }
  return randomness;
}, "randomnessProcess");
var unsafedGetRandomness = /* @__PURE__ */ __name((opts) => {
  return randomnessProcess({
    randomnessLength: opts.randomnessLength,
    charList: opts.charList
  });
}, "unsafedGetRandomness");
var getRandomness = /* @__PURE__ */ __name((opts) => {
  validateNumber({
    name: "RandomnessLength",
    number: opts.randomnessLength
  });
  return randomnessProcess({
    randomnessLength: opts.randomnessLength,
    charList: opts.charList
  });
}, "getRandomness");

// src/functions/common/generate.ts
var generate = /* @__PURE__ */ __name((opts) => {
  try {
    const encoded = encode({
      charList: opts.charList,
      timestamp: opts.timestamp
    });
    const randomness = getRandomness({
      charList: opts.charList,
      randomnessLength: opts.randomnessLength
    });
    return {
      success: true,
      result: encoded + randomness
    };
  } catch (e) {
    return {
      success: false,
      error: e instanceof Error ? e : new Error(String(e))
    };
  }
}, "generate");

// src/functions/common/rowid.ts
var RowID = /* @__PURE__ */ __name((opts) => {
  validateNumber({
    name: "RandomnessLength",
    number: opts.randomnessLength
  });
  return unsafedEncode({
    charList: opts.charList,
    timestamp: (/* @__PURE__ */ new Date()).getTime()
  }) + unsafedGetRandomness({
    charList: opts.charList,
    randomnessLength: opts.randomnessLength
  });
}, "RowID");

// src/functions/common/verify.ts
var verify = /* @__PURE__ */ __name((opts) => {
  try {
    const result = decode({
      charList: opts.charList,
      encoded: opts.encoded
    });
    let natural = true;
    if (result.getTime() > Date.now()) {
      natural = false;
    }
    return {
      success: true,
      result,
      natural
    };
  } catch (e) {
    return {
      success: false,
      error: e instanceof Error ? e : new Error(String(e))
    };
  }
}, "verify");

// src/base/rowid.ts
var RowID2 = /* @__PURE__ */ __name((randomnessLength = RANDOMNESS_LENGTH) => {
  return RowID({
    charList: CHAR_LIST,
    randomnessLength
  });
}, "RowID");
var encode2 = /* @__PURE__ */ __name((timestamp) => {
  return encode({
    charList: CHAR_LIST,
    timestamp
  });
}, "encode");
var decode2 = /* @__PURE__ */ __name((encoded) => {
  return decode({
    charList: CHAR_LIST,
    encoded
  });
}, "decode");
var generate2 = /* @__PURE__ */ __name((timestamp, randomnessLength = RANDOMNESS_LENGTH) => {
  return generate({
    charList: CHAR_LIST,
    timestamp,
    randomnessLength
  });
}, "generate");
var verify2 = /* @__PURE__ */ __name((encoded) => {
  return verify({
    charList: CHAR_LIST,
    encoded
  });
}, "verify");
var getRandomness2 = /* @__PURE__ */ __name((randomnessLength) => {
  return getRandomness({
    charList: CHAR_LIST,
    randomnessLength
  });
}, "getRandomness");

// src/base/rowidWithConfig.ts
var RowIDWithConfig = /* @__PURE__ */ __name((options) => {
  if (options) {
    if (typeof options !== "object") {
      throw new TypeError("Options is not an object");
    }
    if (options.charList) {
      if (typeof options.charList !== "string") {
        throw new TypeError("CharList is not a string");
      }
      if (options.charList.length < 28) {
        throw new RangeError(`CharList's length must be greater than or equal to 28`);
      }
    }
    if (options.randomnessLength) {
      validateNumber({
        name: "RandomnessLength",
        number: options.randomnessLength
      });
    }
  }
  const charList = options.charList ?? CHAR_LIST;
  const _randomnessLength = options.randomnessLength ?? RANDOMNESS_LENGTH;
  return {
    /**
     * This function generates a unique ID
     * that is almost impossible to duplicate.
     *
     * Or you can specify the number of randomness,
     * a larger number will generate a longer ID,
     * with less chance of collision.
     *
     * #### Example
     *
     * ```typescript
     * import type { RowIDWithConfigResult } from "rowid";
     * import { RowIDWithConfig } from "rowid";
     *
     * const { RowID }: RowIDWithConfigResult = RowIDWithConfig({
     *     // ...
     * });
     *
     * const id: string = RowID();
     * ```
     */
    RowID: /* @__PURE__ */ __name((randomnessLength = _randomnessLength) => RowID({
      charList,
      randomnessLength
    }), "RowID"),
    /**
     * This function encodes the timestamp into a ID without randomness.
     *
     * #### Example
     *
     * ```typescript
     * import type { RowIDWithConfigResult } from "rowid";
     * import { RowIDWithConfig } from "rowid";
     *
     * const { encode }: RowIDWithConfigResult = RowIDWithConfig({
     *     // ...
     * });
     *
     * const encoded: string = encode(new Date().getTime());
     * ```
     */
    encode: /* @__PURE__ */ __name((timestamp) => encode({
      charList,
      timestamp
    }), "encode"),
    /**
     * This function decodes the ID into a Date.
     *
     * #### Example
     *
     * ```typescript
     * import type { RowIDWithConfigResult } from "rowid";
     * import { RowIDWithConfig } from "rowid";
     *
     * const { decode }: RowIDWithConfigResult = RowIDWithConfig({
     *     // ...
     * });
     *
     * const decoded: Date = decode("0123456789");
     * ```
     */
    decode: /* @__PURE__ */ __name((encoded) => decode({
      charList,
      encoded
    }), "decode"),
    /**
     * This function generates a ID based on the input.
     *
     * #### Example
     *
     * ```typescript
     * import type { RowIDWithConfigResult, GenerateResult } from "rowid";
     * import { RowIDWithConfig } from "rowid";
     *
     * const { generate }: RowIDWithConfigResult = RowIDWithConfig({
     *     // ...
     * });
     *
     * const current: number = new Date().getTime();
     * const result: GenerateResult = generate(current, 32 - 10);
     * ```
     */
    generate: /* @__PURE__ */ __name((timestamp, randomnessLength = _randomnessLength) => generate({
      charList,
      timestamp,
      randomnessLength
    }), "generate"),
    /**
     * This function verifies if the ID is valid and natural.
     *
     * #### Example
     *
     * ```typescript
     * import type { RowIDWithConfigResult, VerifyResult } from "rowid";
     * import { RowIDWithConfig } from "rowid";
     *
     * const { verify }: RowIDWithConfigResult = RowIDWithConfig({
     *     // ...
     * });
     *
     * const result: VerifyResult = verify("0123456789");
     * ```
     */
    verify: /* @__PURE__ */ __name((encoded) => verify({
      charList,
      encoded
    }), "verify"),
    /**
     * This function generates randomness with different methods based on the environment.
     *
     * #### Example
     *
     * ```typescript
     * import type { RowIDWithConfigResult } from "rowid";
     * import { RowIDWithConfig } from "rowid";
     *
     * const { getRandomness }: RowIDWithConfigResult = RowIDWithConfig({
     *     // ...
     * });
     *
     * const result: string = getRandomness(6);
     * ```
     */
    getRandomness: /* @__PURE__ */ __name((randomnessLength) => getRandomness({
      charList,
      randomnessLength
    }), "getRandomness")
  };
}, "RowIDWithConfig");

// src/index.ts
var src_default = RowID2;
